from django.shortcuts import render, get_object_or_404
from django.contrib.auth.decorators import login_required
from django.utils import timezone
from django.db.models.fields import DateTimeField
from django.db.models import F, Count, Q, ExpressionWrapper
from .models import ShiftRegistration, Room, Shift, Helper
from django.views.generic import DetailView, ListView
from django.contrib.auth.mixins import LoginRequiredMixin

# Create your views here.
def index(request):
    pass

@login_required
def shift_overview(request):
    context = {}
    context['running_shifts'] = (reg.shift for reg in ShiftRegistration.objects.annotate(shift_end=ExpressionWrapper(F('shift__start_at')+F('shift__duration'), output_field=DateTimeField())).filter(shift__start_at__lte=timezone.now(), shift_end__gte=timezone.now()).order_by('shift__start_at'))
    
    # probably can do some distinct/group by stuff but not sure how tih django queries
    context['next_shifts'] = (next(iter(Shift.objects.filter(room=room, start_at__gt=timezone.now()).order_by('start_at'))) for room in Room.objects.all())
    
    return render(request, 'shift_overview.html', context)

class ShiftDetail(LoginRequiredMixin, DetailView):
    template_name = "shift_detail.html"
    model= Shift

class HelperDetail(LoginRequiredMixin, DetailView):
    template_name = "helper_detail.html"
    model= Helper

class ShiftList(LoginRequiredMixin, ListView):
    template_name = "shift_list.html"
    model= Shift